/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package eric.GUI.ZDialog;

import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JButton;
import javax.swing.JLabel;
import rene.gui.Global;

/**
 *
 * @author erichake
 */
public class ZButton extends JButton {

        public void paint(Graphics g) {
                super.paintComponent(g);
        }

        public ZButton(String lbl) {
            super(lbl);
            setOpaque(false);
            setFont(new Font(Global.GlobalFont, 0, 11));
            addMouseListener(new MouseAdapter() {

                @Override
                public void mouseClicked(MouseEvent e) {
                    action();
                }
            });
        }

        public void action() {
        }
    }
