/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package eric.GUI.window;

import java.util.Calendar;

/**
 *
 * @author erichake
 */
public class clock {
    private static long time;

//    public static void start(){
//        time=Calendar.getInstance().getTimeInMillis();
//        System.out.println("-- start --");
//    }
//    public static void stop(){
//        System.out.println((Calendar.getInstance().getTimeInMillis()-time)+" ms");
//    }
//    public static void print(String message){
//        long newtime=Calendar.getInstance().getTimeInMillis();
//        System.out.println(message+" : "+(newtime-time));
//        time=newtime;
//    }
}
