/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package eric.GUI.window;


import eric.JZirkelCanvas;

/**
 *
 * @author erichake
 */
public class tab_canvas_panel extends JZirkelCanvas {
    private tab_btn BTN;

    public tab_canvas_panel(tab_btn btn) {
        super();
        BTN=btn;
    }




}
