/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package eric.JSprogram;



/**
 *
 * @author erichake
 */
public class Const{
    public double BLA=3.5;
public Const(){

}
public double getBLA(){
    return BLA;
}

public double m(String a,String b){
    return 1.0;
}

public double m(String a){
    return 2.0;
}
 
}
