/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package eric.GUI.window;

/**
 *
 * @author erichake
 */
public interface LeftPanel_btn {
public void select(boolean b);
public boolean isPanelSelected();
}
