/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package eric.GUI.window;

import eric.GUI.themes;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenuItem;

/**
 *
 * @author erichake
 */
public class myJMenuItem extends JMenuItem implements ActionListener {
    private boolean selected;

    public void action() {      
    }
    public myJMenuItem(String s) {
        super(s);
        addActionListener(this);
        setFont(themes.TabMenusFont);
    }
    public void actionPerformed(ActionEvent e) {
        action();
    }
            @Override
        public void setSelected(final boolean sel) {
            selected=sel;
            final int fontstyle=(selected)?1:0;

            setFont(new java.awt.Font(themes.TabMenusFont.getFontName(), fontstyle, themes.TabMenusFont.getSize()));
            repaint();
        }
}
