/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package eric.JSprogram;

import java.awt.*;
    import java.io.*;
    import javax.swing.*;

    public class JSOuputConsole extends JFrame {
        JTextArea textArea = new JTextArea();

        public JSOuputConsole() {
            // Add a scrolling text area
            textArea.setEditable(false);
            textArea.setRows(20);
            textArea.setColumns(50);
            getContentPane().add(new JScrollPane(textArea), BorderLayout.CENTER);
            pack();
            setVisible(false);

            // Create reader threads
//            new ReaderThread(piOut).start();
        }

        public void print(String msg){
            textArea.append(msg);
        }

        public void println(String msg){
            textArea.append("\n"+msg);
        }

        

    }