/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package eric.macros;

/**
 *
 * @author erichake
 */
public interface TopDialog {
public void exit();
}
